package ciao_lwt

  1. Overview
  2. Docs

Changelog

All notable changes to this project will be documented in this file.

The format is based on Keep a Changelog, and this project adheres to Semantic Versioning.

0.2

  • Compatibility with OCaml 5.4 and 5.5
  • Generate Promise.await for bound expression that are not function applications. For example,

      let x = async_operation () in
    let* y = x in
    ..

    is transformed to:

      let x = async_operation () in
    let y = Promise.await x in
    ..

    This will likely result in a compiler error, that can easily be fixed instead of a silent change in the program behavior.

0.1

Initial release