package ciao_lwt

  1. Overview
  2. Docs

Module Ocaml_shape_utilsSource

Sourcemodule Decl : sig ... end
Sourcemodule Shap : sig ... end

Not named Shape to avoid confusion with the module from merlin's lib, which is also used elsewhere.

Sourcemodule Def_to_decl : sig ... end

Map definition UIDs (from the interface) to declaration UIDs (from the implementation), that can be looked up in indexes.

Sourcetype cmt = {
  1. unit_name : string;
  2. path : string;
  3. decls : Decl.t list;
  4. intf : Ocaml_typing.Cmi_format.cmi_infos option;
    (*

    None if a .mli is present, unless ~read_cmti:true is passed.

    *)
  5. shape : Shap.t;
  6. def_to_decl : Def_to_decl.t;
}
Sourceval cmts_of_packages : packages:string list -> units:(string -> bool) -> cmt list

Path to the .cmt in the given packages. Cmts for which units "Unit_name" return false are not collected. Uses ocamlfind.

Sourceval cmt_of_path : ?read_cmti:bool -> Fpath.t -> cmt option

If ~read_cmti is true (defaults to false), the corresponding cmti file in the same directory is also read.

Sourceval pp : Format.formatter -> cmt -> unit